'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (options) {

  var manifest = {};
  //use assert to require filesPath, outputDir
  (0, _assert2.default)(options.files, 'files property is required');
  var filesPath = options.files;
  var baseStr = options.baseStr || '';
  var outputDir = options.outputDir || __dirname;
  var outputDest = _path2.default.resolve(outputDir);
  var file = options.file;
  var hash = options.hash || false;

  function writeManifest(manifest) {
    if (file) {
      _fsExtra2.default.ensureFileSync(_path2.default.resolve(file));
      _fsExtra2.default.writeFileSync(_path2.default.resolve(file), JSON.stringify(manifest), 'utf8');
    } else {
      _fsExtra2.default.writeFileSync(_path2.default.join(__dirname, 'assets.json'), JSON.stringify(manifest), 'utf8');
    }
  }

  var filesPathParts = filesPath.split(',');
  var files = [];
  filesPathParts.forEach(function (filePathPart) {
    files = files.concat(files, _glob2.default.sync(_path2.default.resolve(filePathPart), {}));
  });
  // Uniquify files
  files = Array.from(new Set(files));

  var baseDir = void 0;
  if (files && files.length === 1) {
    baseDir = files[0].split(_path2.default.sep).slice(0, -1).join(_path2.default.sep);
  } else {
    baseDir = (0, _commondir2.default)(files);
  }
  if (files && files.length) {
    var replaceStringsFor = function replaceStringsFor(target) {
      // Check if target is a dependent
      if (dependencyMap[target]) {
        for (var dependency in dependencyMap[target]) {
          if (dependency.match(/\.(css|js|html)$/i)) {
            // Target to be recalculated
            if (typeof timesInRecursion[dependency + "==" + target] === "undefined") {
              timesInRecursion[dependency + "==" + target] = 0;
            }
            // Check if dependecy loop
            if (timesInRecursion[dependency + "==" + target] < 100) {
              timesInRecursion[dependency + "==" + target] = parseInt(timesInRecursion[dependency + "==" + target]) + 1;
              replaceStringsFor(dependency);
            } else {
              console.log("Too deep recursion in dependencies for: [ " + dependency + " ] included in: [ " + target + " ]");
              delete dependencyMap[target][dependency];
            }
          }
        }
      }
      // // First find the file in the array
      for (var f in manifest) {
        // File ref found!
        if (f === target) {
          var contents = _fsExtra2.default.readFileSync(_path2.default.resolve(_path2.default.join(outputDest, manifest[f]))).toString();
          var dependencies = Object.keys(dependencyMap[f]).sort(function (a, b) {
            return b.length - a.length;
          });
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = dependencies[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var depToReplace = _step.value;

              contents = contents.replace(new RegExp(depToReplace, 'g'), manifest[depToReplace]);
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          var h = (0, _revHash2.default)(Buffer.from(contents));
          var newPath = (0, _revPath2.default)(f, h);

          _fsExtra2.default.writeFileSync(_path2.default.resolve(_path2.default.join(outputDest, newPath)), contents);
          manifest[f] = newPath;
          break;
        }
      }
    };

    files.forEach(function (file) {
      var parsedPath = _path2.default.parse(file);
      var filename = parsedPath.base;
      var dirParts = parsedPath.dir.split('/');
      var fileDirParts = [];
      while (dirParts.join('/') !== baseDir) {
        fileDirParts.unshift(dirParts.pop());
      }
      fileDirParts.unshift(dirParts.pop());
      fileDirParts = fileDirParts.filter(function (el) {
        return !baseStr.split('/').includes(el);
      });
      var fileDir = fileDirParts.join('/');
      var buffer = _fsExtra2.default.readFileSync(file);
      if (hash) {
        var h = (0, _revHash2.default)(buffer);
        var revdPath = (0, _revPath2.default)(_path2.default.join(fileDir, filename), h);
        manifest[_path2.default.join(fileDir, filename)] = revdPath;
        _fsExtra2.default.ensureFileSync(_path2.default.join(outputDest, revdPath));
        _fsExtra2.default.copyFileSync(file, _path2.default.join(outputDest, revdPath));
      } else {
        manifest[_path2.default.join(fileDir, filename)] = _path2.default.join(fileDir, filename);
        _fsExtra2.default.ensureFileSync(_path2.default.join(outputDest, _path2.default.join(fileDir, filename)));
        _fsExtra2.default.copyFileSync(file, _path2.default.join(outputDest, fileDir, filename));
      }
    });

    var dependencyMap = {};
    if (hash) {
      Object.keys(manifest).forEach(function (f) {
        if (f.match(/\.(css|js|html)$/i)) {
          var content = _fsExtra2.default.readFileSync(_path2.default.resolve(_path2.default.join(outputDest, manifest[f]))).toString();
          Object.keys(manifest).forEach(function (k) {
            if (content.match(k)) {
              dependencyMap[f] = dependencyMap[f] || {};
              dependencyMap[f][k] = true;
            }
          });
        }
      });
    }

    var timesInRecursion = {};
    ;

    // Fix recursive hashes
    if (hash) {
      for (var dependent in dependencyMap) {
        replaceStringsFor(dependent);
      }
    }

    writeManifest(manifest);
  } else {
    console.warn('No files found matching ' + _path2.default.resolve(filesPath));
    writeManifest({});
  }
};

var _revHash = require('rev-hash');

var _revHash2 = _interopRequireDefault(_revHash);

var _revPath = require('rev-path');

var _revPath2 = _interopRequireDefault(_revPath);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _commondir = require('commondir');

var _commondir2 = _interopRequireDefault(_commondir);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
//# sourceMappingURL=index.js.map